import { mkdirSync } from 'fs';
import { dirname, join, resolve, sep } from 'path';
import { fileURLToPath } from 'url';
import chalk from 'chalk';
const createFolderFor = (filename) => mkdirSync(dirname(filename), { recursive: true });
const logWarn = (...text) => console.log(chalk.yellow(...text)); // eslint-disable-line no-console
const createGetMyConfig = (pluginConfig, pluginFunctionStore) => (plugin) => {
    const name = pluginFunctionStore.get(plugin);
    return name === undefined ? {} : pluginConfig.get(name) ?? {};
};
const createRegisterPlugin = (pluginFunctionStore, plugins, scullyConfig) => (type, name, plugin, priority = 100) => {
    const sanitizedType = type === 'render' ? 'postProcessByHtml' : type;
    if (sanitizedType === 'postProcessByHtml') {
        let pluginsOfType = plugins.get(sanitizedType);
        if (pluginsOfType === undefined) {
            pluginsOfType = new Map();
            plugins.set(sanitizedType, pluginsOfType);
        }
        let pluginsWithSamePriority = pluginsOfType.get(priority);
        if (pluginsWithSamePriority === undefined) {
            pluginsWithSamePriority = [];
            pluginsOfType.set(priority, pluginsWithSamePriority);
        }
        pluginsWithSamePriority.push([
            name,
            (partialConfig, ...args) => {
                Object.assign(scullyConfig, partialConfig);
                return plugin(...args);
            }
        ]);
    }
    else {
        let pluginsOfType = plugins.get(sanitizedType);
        if (pluginsOfType === undefined) {
            pluginsOfType = new Map();
            plugins.set(sanitizedType, pluginsOfType);
        }
        let pluginsWithSamePriority = pluginsOfType.get(priority);
        if (pluginsWithSamePriority === undefined) {
            pluginsWithSamePriority = [];
            pluginsOfType.set(priority, pluginsWithSamePriority);
        }
        pluginsWithSamePriority.push([
            name,
            (partialConfig, ...args) => {
                Object.assign(scullyConfig, partialConfig);
                return plugin(...args);
            }
        ]);
    }
    pluginFunctionStore.set(plugin, name);
};
const createSetPluginConfig = (pluginConfig) => (name, config) => pluginConfig.set(name, config);
export const loadScullyConfigAndPlugins = async (cwd, require, // eslint-disable-line no-undef
scullyConfigFile) => {
    if (scullyConfigFile === undefined) {
        return { config: null, plugins: null };
    }
    const filename = require.resolve('@scullyio/scully', { paths: [cwd] });
    const paths = [];
    let path = dirname(filename);
    while (path !== sep) {
        paths.push(join(path, 'node_modules'));
        path = dirname(path);
    }
    paths.push(join(path, 'node_modules'));
    const originalCache = require.cache[filename];
    const pluginConfigStore = new Map();
    const pluginFunctionStore = new WeakMap();
    const plugins = new Map();
    const scullyConfig = { defaultPostRenderers: [], distFolder: '', outDir: '' };
    require.cache[filename] = {
        children: [],
        exports: {
            createFolderFor,
            getMyConfig: createGetMyConfig(pluginConfigStore, pluginFunctionStore),
            logWarn,
            registerPlugin: createRegisterPlugin(pluginFunctionStore, plugins, scullyConfig),
            scullyConfig,
            setPluginConfig: createSetPluginConfig(pluginConfigStore)
        },
        filename,
        id: filename,
        isPreloading: false,
        loaded: true,
        parent: require.cache[dirname(fileURLToPath(import.meta.url))],
        path: paths[0],
        paths,
        require
    };
    const { config } = await import(resolve(scullyConfigFile));
    if (originalCache === undefined) {
        delete require.cache[filename]; // eslint-disable-line @typescript-eslint/no-dynamic-delete
    }
    else {
        require.cache[filename] = originalCache;
    }
    return { config: { ...scullyConfig, ...config }, plugins };
};
//# sourceMappingURL=load-scully-config-and-plugins.js.map